/**
 * 
 */
package com.atlassian.crucible.plugin.export;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Locale;

import org.junit.Before;
import org.junit.Test;

import com.atlassian.crucible.plugin.export.exception.ExportException;
import com.atlassian.crucible.plugin.export.exception.ReviewDataRetrievalException;
import com.atlassian.crucible.spi.PermId;
import com.atlassian.crucible.spi.data.FisheyeReviewItemData;
import com.atlassian.crucible.spi.data.GeneralCommentData;
import com.atlassian.crucible.spi.data.ReviewData;
import com.atlassian.crucible.spi.data.ReviewItemData;
import com.atlassian.crucible.spi.data.VersionedLineCommentData;
import com.atlassian.crucible.spi.data.ReviewData.State;

/**
 * Performs validation assertions on the {@link ReviewExporter} class.
 * 
 * @author Ross Rowe
 * 
 */
public class ReviewExporterTest {

	private static final String BASE_URL = "http://localhost:8060";

	private static final String TEST_PDF_NAME = "test.pdf";

	private ReviewExporter reviewExporter;

	private static final String VALID_USER_NAME = "ross";

	private static final String VALID_PASSWORD = "ross";

	private static final String VALID_REVIEW_ID = "CR-1";
	
	/**
	 * Used for logic testing.
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		ReviewExporter reviewExporter = new ReviewExporter(
				new CrucibleStubClient(), Locale.getDefault());
		reviewExporter.setBaseUrl("http://localhost:8060");
		ByteArrayOutputStream outputStream = reviewExporter.createPdfExport(
				"ross", "ross", "CR-2");
		FileOutputStream fileStream = new FileOutputStream(new File("test.pdf"));
		fileStream.write(outputStream.toByteArray());
		fileStream.close();
	}

	@Before
	public void setUp() {
		CrucibleClient client = new CrucibleStubClient();
		reviewExporter = new ReviewExporter(client, Locale.getDefault());
		reviewExporter.setBaseUrl(BASE_URL);
		File pdfFile = new File(TEST_PDF_NAME);
		if (pdfFile.exists())
			pdfFile.delete();
	}

	@Test(expected = ReviewDataRetrievalException.class)
	public void nullUserName() throws Exception {
		reviewExporter.createPdfExport(null, VALID_PASSWORD, VALID_REVIEW_ID);
	}

	@Test(expected = ReviewDataRetrievalException.class)
	public void nullPassword() throws Exception {
		reviewExporter.createPdfExport(VALID_USER_NAME, null, VALID_REVIEW_ID);
	}

	@Test(expected = ReviewDataRetrievalException.class)
	public void nullReviewId() throws Exception {
		reviewExporter.createPdfExport(VALID_USER_NAME, VALID_USER_NAME, null);
	}

	@Test(expected = ReviewDataRetrievalException.class)
	public void emptyUserName() throws Exception {
		reviewExporter.createPdfExport("", VALID_PASSWORD, VALID_REVIEW_ID);
	}

	@Test(expected = ReviewDataRetrievalException.class)
	public void emptyPassword() throws Exception {
		reviewExporter.createPdfExport(VALID_USER_NAME, "", VALID_REVIEW_ID);
	}

	@Test(expected = ReviewDataRetrievalException.class)
	public void emptyReviewId() throws Exception {
		reviewExporter.createPdfExport(VALID_USER_NAME, VALID_USER_NAME, "");
	}

	@Test(expected = ReviewDataRetrievalException.class)
	public void nullReviewData() throws Exception {
		reviewExporter.createPdfExport(null, null, null);
	}

	@Test
	public void validReview() throws Exception {
		File pdfFile = new File(TEST_PDF_NAME);
		assertFalse(pdfFile.exists());
		runValidExport(pdfFile);

	}
	
	@Test
	public void validReviewWithDummyData() throws Exception {
		File pdfFile = new File(TEST_PDF_NAME);
		assertFalse(pdfFile.exists());
		
		CrucibleClient client = reviewExporter.getClient();
		GeneralCommentData generalComment = new GeneralCommentData();
		generalComment.setPermaId(new PermId<GeneralCommentData>("1"));
		client.getGeneralComments(null).add(generalComment);
		PermId<ReviewItemData> reviewItemId = new PermId<ReviewItemData>("1");
		PermId<VersionedLineCommentData> commentId = new PermId<VersionedLineCommentData>("1");
		addVersionedComment(client, reviewItemId, commentId);
		
		ReviewData reviewData = client.getReview(null);
		reviewData.setState(State.Draft);
		
		FisheyeReviewItemData reviewItemData = client.getReviewItemData(null, null);
		reviewItemData.setPermId(reviewItemId);
		reviewItemData.setToPath("test.txt");
		runValidExport(pdfFile);
	}
	
	@Test
	public void draftComment() throws Exception {
		File pdfFile = new File(TEST_PDF_NAME);
		assertFalse(pdfFile.exists());
		
		CrucibleClient client = reviewExporter.getClient();
		GeneralCommentData generalComment = new GeneralCommentData();
		generalComment.setPermaId(new PermId<GeneralCommentData>("1"));
		client.getGeneralComments(null).add(generalComment);
		PermId<ReviewItemData> reviewItemId = new PermId<ReviewItemData>("1");
		PermId<VersionedLineCommentData> commentId = new PermId<VersionedLineCommentData>("1");
		addDraftVersionedComment(client, reviewItemId, commentId);
		
		ReviewData reviewData = client.getReview(null);
		reviewData.setState(State.Draft);
		
		FisheyeReviewItemData reviewItemData = client.getReviewItemData(null, null);
		reviewItemData.setPermId(reviewItemId);
		reviewItemData.setToPath("test.txt");
		runValidExport(pdfFile);
	}
	
	@Test
	public void defectComment() throws Exception {
		File pdfFile = new File(TEST_PDF_NAME);
		assertFalse(pdfFile.exists());
		
		CrucibleClient client = reviewExporter.getClient();
		GeneralCommentData generalComment = new GeneralCommentData();
		generalComment.setPermaId(new PermId<GeneralCommentData>("1"));
		client.getGeneralComments(null).add(generalComment);
		PermId<ReviewItemData> reviewItemId = new PermId<ReviewItemData>("1");
		PermId<VersionedLineCommentData> commentId = new PermId<VersionedLineCommentData>("1");
		addDefectVersionedComment(client, reviewItemId, commentId);
		
		ReviewData reviewData = client.getReview(null);
		reviewData.setState(State.Draft);
		
		FisheyeReviewItemData reviewItemData = client.getReviewItemData(null, null);
		reviewItemData.setPermId(reviewItemId);
		reviewItemData.setToPath("test.txt");
		runValidExport(pdfFile);
	}
	
	private void addDefectVersionedComment(CrucibleClient client,
			PermId<ReviewItemData> reviewItemId,
			PermId<VersionedLineCommentData> commentId) {
		VersionedLineCommentData versionedComment = addVersionedComment(client, reviewItemId, commentId);
		versionedComment.setDefectRaised(true);
	}

	@Test
	public void twoCommentsForSingleFile() throws Exception {
		File pdfFile = new File(TEST_PDF_NAME);
		assertFalse(pdfFile.exists());
		
		CrucibleClient client = reviewExporter.getClient();
		GeneralCommentData generalComment = new GeneralCommentData();
		generalComment.setPermaId(new PermId<GeneralCommentData>("1"));
		client.getGeneralComments(null).add(generalComment);
		PermId<ReviewItemData> reviewItemId = new PermId<ReviewItemData>("1");
		PermId<VersionedLineCommentData> commentId = new PermId<VersionedLineCommentData>("1");
		PermId<VersionedLineCommentData> secondCommentId = new PermId<VersionedLineCommentData>("10");
		addVersionedComment(client, reviewItemId, commentId);
		addDraftVersionedComment(client, reviewItemId, secondCommentId);
		
		ReviewData reviewData = client.getReview(null);
		reviewData.setState(State.Draft);
		
		FisheyeReviewItemData reviewItemData = client.getReviewItemData(null, null);
		reviewItemData.setPermId(reviewItemId);
		reviewItemData.setToPath("test.txt");
		
		runValidExport(pdfFile);

	}

	private VersionedLineCommentData addVersionedComment(CrucibleClient client,
			PermId<ReviewItemData> reviewItemId,
			PermId<VersionedLineCommentData> commentId) {
		VersionedLineCommentData versionedComment = new VersionedLineCommentData();
		versionedComment.setReviewItemId(reviewItemId);
		versionedComment.setPermaId(commentId);
		versionedComment.setDraft(false);
		client.getVersionedComments(null).add(versionedComment);
		return versionedComment;
	}
	
	private void addDraftVersionedComment(CrucibleClient client,
			PermId<ReviewItemData> reviewItemId,
			PermId<VersionedLineCommentData> commentId) {
		VersionedLineCommentData versionedComment = addVersionedComment(client, reviewItemId, commentId);
		versionedComment.setDraft(true);
	}
	
	@Test
	public void commentWithNullReviewItemId() throws Exception {
		File pdfFile = new File(TEST_PDF_NAME);
		assertFalse(pdfFile.exists());
		
		CrucibleClient client = reviewExporter.getClient();
		GeneralCommentData generalComment = new GeneralCommentData();
		generalComment.setPermaId(new PermId<GeneralCommentData>("1"));
		client.getGeneralComments(null).add(generalComment);
		PermId<ReviewItemData> reviewItemId = new PermId<ReviewItemData>("1");
		PermId<VersionedLineCommentData> commentId = new PermId<VersionedLineCommentData>("1");
		addVersionedComment(client, reviewItemId, commentId);
		
		ReviewData reviewData = client.getReview(null);
		reviewData.setState(State.Draft);
		
		FisheyeReviewItemData reviewItemData = client.getReviewItemData(null, null);
		reviewItemData.setPermId(reviewItemId);
		reviewItemData.setToPath("test.txt");
		
		runValidExport(pdfFile);

	}
	
	@Test
	public void commentWithNoReviewItem() throws Exception {
		File pdfFile = new File(TEST_PDF_NAME);
		assertFalse(pdfFile.exists());
		
		CrucibleClient client = reviewExporter.getClient();
		GeneralCommentData generalComment = new GeneralCommentData();
		generalComment.setPermaId(new PermId<GeneralCommentData>("1"));
		client.getGeneralComments(null).add(generalComment);
		VersionedLineCommentData versionedComment = new VersionedLineCommentData();
		PermId<ReviewItemData> reviewItemId = new PermId<ReviewItemData>("1");
		PermId<ReviewItemData> commentReviewItemId = new PermId<ReviewItemData>("1");
		versionedComment.setReviewItemId(commentReviewItemId);
		versionedComment.setPermaId(new PermId<VersionedLineCommentData>("10"));
		client.getVersionedComments(null).add(versionedComment);
		
		ReviewData reviewData = client.getReview(null);
		reviewData.setState(State.Draft);
		
		FisheyeReviewItemData reviewItemData = client.getReviewItemData(null, null);
		reviewItemData.setPermId(reviewItemId);
		reviewItemData.setToPath("test.txt");
		
		runValidExport(pdfFile);

	}

	private void runValidExport(File pdfFile) throws ExportException,
			FileNotFoundException, IOException {
		ByteArrayOutputStream outputStream = reviewExporter.createPdfExport(
				"ross", "ross", "CR-2");
		FileOutputStream fileStream = new FileOutputStream(pdfFile);
		fileStream.write(outputStream.toByteArray());
		fileStream.close();
		assertTrue(pdfFile.exists());
	}
}
